USE HashMatch;
GO

-- Perform a Left Outer Join, using Hash Match
-- The CTEs with TOP force the inputs to be in a known order.

-- In this case (and in most other demos for this session),
-- I use hints to force both hash match and a serial plan;
-- this is not a good idea for production code but easier for demos.

-- I force the build input to be in a known order.
-- I then use LEFT JOIN to an empty probe input to ensure no matches
-- The entire build input is unmatched, and is returned after the probe phase (shown in a later demo).
-- This allows us to see the order of rows that are in the same bucket (same Key1 value, or hash collission)
-- ---> Some interesting points in the output (SQL Server 2017):
--      * 1 value  in bucket: 0 (rows 1 - 5)
--      * 2 values in bucket: 23, 687 (rows 6 - 15)
--      * 3 values in bucket: 46, 215, 710 (rows 16 - 30)
--      * 4 values in bucket: 32, 364, 573, 659 (rows 906 - 925)
-- ---> Some interesting points in the output (SQL Server 2016):
--      * 1 value  in bucket: 0 (rows 1 - 5)
--      * 2 values in bucket: 501, 756 (rows 71 - 80)
--      * 3 values in bucket: 23, 355, 687 (rows 6 - 20)
--      * 4 values in bucket: 109, 607, 650, 773 (rows 171 - 190)
--      * 5 values in bucket: 46, 215, 378, 547, 710 (rows 21 - 45)
-- (found using the second query)
WITH BuildTop AS
   (SELECT TOP (10000) *
    FROM        dbo.BuildInput
    WHERE       Key1 <= 799
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT      *
    FROM        dbo.ProbeInput
    WHERE       Key1 < 0)
SELECT      b.Key1, b.Key2, p.Key1, p.Key2
FROM        BuildTop AS b
LEFT  JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- The same query, except the order of the build input is now different
-- ---> Some interesting points in the output (SQL Server 2017):
--      * 1 value  in bucket: 0 (rows 1 - 5)
--      * 2 values in bucket: 23, 687 (rows 6 - 15)
--      * 3 values in bucket: 46, 215, 710 (rows 16 - 30)
--      * 4 values in bucket: 32, 364, 573, 659 (rows 906 - 925)
-- (no occurences with 5 or more values in bucket observed)
-- ---> Some interesting points in the output (SQL Server 2016):
--      * 1 value  in bucket: 0 (rows 1 - 5)
--      * 2 values in bucket: 501, 756 (rows 71 - 80)
--      * 3 values in bucket: 23, 355, 687 (rows 6 - 20)
--      * 4 values in bucket: 109, 607, 650, 773 (rows 171 - 190)
--      * 5 values in bucket: 46, 215, 378, 547, 710 (rows 21 - 45)
-- (no occurences with 6 or more values in bucket observed)
WITH BuildTop AS
   (SELECT TOP (10000) *
    FROM        dbo.BuildInput
    WHERE       Key1 <= 799
    ORDER BY    Key2, Key1)
, ProbeTop AS
   (SELECT      *
    FROM        dbo.ProbeInput
    WHERE       Key1 < 0)
SELECT      b.Key1, b.Key2, p.Key1, p.Key2
FROM        BuildTop AS b
LEFT  JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);
