USE HashMatch;
GO

-- For Right Anti Semi Join and Right Semi Join, we can use the same basic scripts as for the the Left (Anti) Semi Joins.
-- However, the optimizer favors Left over Right for these (in this case - see note below), so it needs some additional convincing.
-- By changing the rowcount, I ensure that the table I want as the build input is smaller. That works.
-- NOTE: Left (Anti) Semi Join is considered cheaper in this case because:
--       (a) both inputs have the same estimated rowcount and rowsize; and
--       (b) the optimizer wants to produce the entire set as fast as possible.
--       When working with a rowgoal < the estimated total result, the optimizer starts favoring Right (Anti) Semi Join,
--       due to that version being only partially blocking, versus Left (Anti) Semi Join being fully blocking.

-- Right Anti Semi Join: Output in order of probe input
WITH BuildTop AS
   (SELECT TOP (100) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (101) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      p.Key1, p.Key2
FROM        ProbeTop AS p
WHERE NOT EXISTS
   (SELECT  *
    FROM    BuildTop AS b
    WHERE   b.Key1 = p.Key1
    AND     b.Key2 = p.Key2)
OPTION (HASH JOIN, MAXDOP 1);




-- Right Semi Join: Output in order of probe input
SELECT TOP (100) *
INTO        #BuildTop
FROM        dbo.BuildInput
ORDER BY    Key1, Key2;

WITH ProbeTop AS
   (SELECT TOP (101) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      p.Key1, p.Key2
FROM        ProbeTop AS p
WHERE EXISTS
   (SELECT  *
    FROM    #BuildTop AS b
    WHERE   b.Key1 = p.Key1
    AND     b.Key2 = p.Key2)
OPTION (HASH JOIN, MAXDOP 1);
GO

DROP TABLE #BuildTop;
GO



-- So, can we also disprove the alternate possible build phase?
-- Not with 100% certainty, but we can see some strong indications

-- Here is a Right Outer Join with a much larger build input.
-- Look at the memory grant to see the memory reserved for the hash table
WITH BuildTop AS
   (SELECT TOP (1000) Key1
    FROM        dbo.BuildInput
    WHERE       Key1 < 500
    ORDER BY    Key2, Key1)
, ProbeTop AS
   (SELECT TOP (10000) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      p.Key1, p.Key2
FROM        BuildTop AS b
RIGHT JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

-- This query uses a Right Anti Semi Join, with the exact same build input.
-- The hash function is on Key1 only, so we expect to have multiple rows per value in the build input
-- If the build phase would eliminate those, then we would need a much lower memory grant
-- Instead, it is exactly the same, which suggests that all rows are stored, without removing duplicates.
WITH BuildTop AS
   (SELECT TOP (1000) Key1
    FROM        dbo.BuildInput
    WHERE       Key1 < 500
    ORDER BY    Key2, Key1)
, ProbeTop AS
   (SELECT TOP (10000) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT     p.Key1, p.Key2
FROM       ProbeTop AS p
WHERE NOT EXISTS
   (SELECT  *
    FROM    BuildTop AS b
    WHERE   b.Key1 = p.Key1)
OPTION (HASH JOIN, MAXDOP 1);


-- Finally, if we use the exact same pattern but with an even larger build input,
-- then we see the optimizer adds an explicit operator to remove the duplicates before creating the hash table
-- (So now we do see a reduced memory footprint, but the reduction requires a separate operator to remove the duplicates)
WITH BuildTop AS
   (SELECT TOP (200000) Key1
    FROM        dbo.BuildInput
    WHERE       Key1 < 50000
    ORDER BY    Key2, Key1)
, ProbeTop AS
   (SELECT TOP (400000) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      p.Key1, p.Key2
FROM        BuildTop AS b
RIGHT JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1);

WITH BuildTop AS
   (SELECT TOP (200000) Key1
    FROM        dbo.BuildInput
    WHERE       Key1 < 50000
    ORDER BY    Key2, Key1)
, ProbeTop AS
   (SELECT TOP (400000) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT     p.Key1, p.Key2
FROM       ProbeTop AS p
WHERE NOT EXISTS
   (SELECT  *
    FROM    BuildTop AS b
    WHERE   b.Key1 = p.Key1)
OPTION (HASH JOIN, MAXDOP 1);
