USE HashMatch;
GO

-- To experiment with memory grants, change the OPTIMIZE FOR hint in the queries.
-- The memory grant is easy to see in an execution plan with run-time statistics,
-- so I keep the actual values of @i and @j low.
-- That way I can execute the query and get the execution plan plus fast,
-- even when computing grant for huge sets.

-- First, let's look at the impact of the size of the hash key.
-- The queries below that this has NO influence on the size of the memory grant (as expected).
DECLARE @i int = 100,
        @j int = 100;

-- Hash key: integer + char(1) column
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      *
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 250000)); 

-- Hash key: Only an integer column
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      *
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 250000)); 

-- Hash key: integer + char(1) + char(1000) column
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      *
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
      AND   b.Filler = p.Filler
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 250000)); 
GO



-- Some logical operations require rows to be marked in the hash table; other don't.
-- Does this impact the memory grant?
-- (Testing with large row count and small row size to maximize chance of seeing the effect)

DECLARE @i int = 100,
        @j int = 100;

-- 1,000,000 rows; char(1) only; inner join (no marking needed)
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key2
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 1000000, @j = 1000000)); 

-- 1,000,000 rows; char(1) only; full outer join (does require marking)
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key2
FROM        BuildTop AS b
FULL  JOIN  ProbeTop AS p
      ON    b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 1000000, @j = 1000000)); 
GO



-- Since the build input needs to be stored in the hash table,
-- I expect that estimated row size will have a big impact on the memory grant
-- (Note that the hash table needs to include both output columns and probe residual columns!)

DECLARE @i int = 100,
        @j int = 100;

-- 10,000 rows; integer + char(1)
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 250000)); 

-- 10,000 rows; only integer
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 250000)); 

-- 10,000 rows; integer + char(1) + char(1000)
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2, b.Filler
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 250000)); 
GO



-- Obviously, the estimated row COUNT of the build input also affects the memory grant.

DECLARE @i int = 100,
        @j int = 100;

-- 100 rows; integer + char(1)
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 100, @j = 250000)); 

-- 100,000 rows; integer + char(1)
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 100000, @j = 250000)); 
GO



-- Surprisingly, the estimated rowcount of the PROBE input also influences the memory grant.
-- Note that this does not imply that the probe input will be stored in the hash table. It won't,
-- The extra memory is most likely to facilitate graceful handling of insufficient memory through
-- the dynamic destaging process (internal buffers for spilling data to tempdb; index structures to
-- map hash values to partitions; and perhaps more margin for handling dynamic role reversal)
-- Another possible explanation is that, because dynamic staging requires writing all or most of
-- the probe input to files in tempdb, a bigger estimated probe input means a bigger estimated impact of
-- dynamic staging. The higher the estimated impact of dynamic staging, the more margin the optimizer wants.

DECLARE @i int = 100,
        @j int = 100;

-- 10,000 rows in build; 100 in probe
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 100)); 

-- 10,000 rows in build; 10,000 in probe
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 10000)); 

-- 10,000 rows in build; 1,000,000 in probe
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 1000000)); 
GO



-- So if estimated row COUNT of probe has effect, how about estimated row SIZE of probe?
-- Apparently, small differences have no effect but adding a large string column does.
-- The possible explanations remain the same as for the impact of the probe's row COUNT.

DECLARE @i int = 100,
        @j int = 100;

-- 250,000 rows; integer + char(1)
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2, p.Key1, p.Key2
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 250000)); 

-- 250,000 rows; only integer
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2, p.Key1
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 250000)); 

-- 250,000 rows; integer + char(1) + char(1000)
WITH BuildTop AS
   (SELECT TOP (@i) *
    FROM        dbo.BuildInput
    ORDER BY    Key1, Key2)
, ProbeTop AS
   (SELECT TOP (@j) *
    FROM        dbo.ProbeInput
    ORDER BY    Key1, Key2)
SELECT      b.Key1, b.Key2, p.Key1, p.Key2, p.Filler
FROM        BuildTop AS b
INNER JOIN  ProbeTop AS p
      ON    b.Key1 = p.Key1
      AND   b.Key2 = p.Key2
OPTION (HASH JOIN, FORCE ORDER, MAXDOP 1,
        OPTIMIZE FOR (@i = 10000, @j = 250000)); 
GO
