-- Script to set up the demo database and fill the demo tables.
-- Notes:
-- 1. Can take a long time to run (on my laptop, it takes less than one minute on SQL Server 2017, but up to
--    10 minutes on SQL Server 2016 - probably because SQL 2016 has tempdb on spinning disk instead of SSD)
-- 2. Allocates demo database in same folder as master database (change if required)
-- 3. Needs enough available disk space (13000 MB)

USE master;
GO

-- Remove old version of demo database. Use brute force if necessary.
IF EXISTS (SELECT * FROM sys.databases WHERE name = N'HashMatch')
  BEGIN;
  ALTER DATABASE HashMatch SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
  DROP DATABASE HashMatch;
  END;
GO

-- Create the empty demo database, with sufficient size for the demo table.
-- To make the script run on all databases, I don't hardcode the directory,
-- but use the directory where the master database lives;
-- this gives me the guarantee that the directory exists
-- and is accessible by SQL Server.
--
-- Unfortunately, this requires the use of dynamic SQL,
-- which I normally avoid. In this case, though, the risk is acceptable.
-- The only way to exploit this dynamic SQL is to install an instance
-- with the master database in a directory with a carefully crafted pathname;
-- whoever can do that must have admin rights
-- and doesn't need the SQL injection attack.
DECLARE @master_directory nvarchar(260);
SET @master_directory =
 (SELECT REPLACE(physical_name, 'master.mdf', '')
  FROM   sys.database_files
  WHERE  name='master'
  AND    type=0);
EXECUTE
 (N'CREATE DATABASE HashMatch
    ON PRIMARY
      (NAME = HashMatch,
       FILENAME = ''' + @master_directory + N'HashMatch.mdf'',
       SIZE = 6000MB,
       FILEGROWTH = 500MB)
    LOG ON
      (NAME = HashMatchLog,
       FILENAME = ''' + @master_directory + N'HashMatch.ldf'',
       SIZE = 7000MB,
       FILEGROWTH = 500MB);');
GO

-- Ensure SIMPLE recovery
ALTER DATABASE HashMatch SET RECOVERY SIMPLE;

-- Switch to tempdb before switching to demo database.
-- If demo database was not created, the USE statement fails,
-- but the next batch will still execute.
-- Switching to tempdb first ensures spurious tables will be created there.

USE tempdb;
GO
USE HashMatch;
GO

-- Set up tables
-- The 1000-char filler column is used to verify memory grant requirements
CREATE TABLE dbo.BuildInput
   (Key1       bigint     NOT NULL,
    Key2       char(1)    NOT NULL,
    Filler     char(1000) NOT NULL DEFAULT (REPLICATE('x', 400)),
    CONSTRAINT PK_BuildInput
        PRIMARY KEY (Key1, Key2));

CREATE TABLE dbo.ProbeInput
   (Key1       bigint     NOT NULL,
    Key2       char(1)    NOT NULL,
    Filler     char(1000) NOT NULL DEFAULT (REPLICATE('x', 400)),
    CONSTRAINT PK_ProbeInput
        PRIMARY KEY (Key1, Key2));

-- Build input: Numbers 0 - 499999, with letters A, B, D, F, G
WITH Digits(d) AS
       (SELECT 0 UNION ALL SELECT 1 UNION ALL SELECT 2 UNION ALL
        SELECT 3 UNION ALL SELECT 4 UNION ALL SELECT 5 UNION ALL
        SELECT 6 UNION ALL SELECT 7 UNION ALL SELECT 8 UNION ALL SELECT 9),
     Numbers(n) AS
       (SELECT     d5.d * 100000 + d4.d * 10000 + d3.d * 1000 + d2.d * 100 + d1.d * 10 + d0.d
        FROM       Digits AS d0
        CROSS JOIN Digits AS d1
        CROSS JOIN Digits AS d2
        CROSS JOIN Digits AS d3
        CROSS JOIN Digits AS d4
        CROSS JOIN Digits AS d5
        WHERE      d5.d <= 4),
     Letters(l) AS
       (SELECT 'A' UNION ALL SELECT 'B' UNION ALL SELECT 'D' UNION ALL SELECT 'F' UNION ALL SELECT 'G')
INSERT INTO dbo.BuildInput
           (Key1, Key2)
SELECT      n.n, l.l
FROM        Numbers AS n
CROSS JOIN  Letters AS l;

-- Probe input: Numbers 0 - 499999, with letters A, C, D, E, G
WITH Digits(d) AS
       (SELECT 0 UNION ALL SELECT 1 UNION ALL SELECT 2 UNION ALL
        SELECT 3 UNION ALL SELECT 4 UNION ALL SELECT 5 UNION ALL
        SELECT 6 UNION ALL SELECT 7 UNION ALL SELECT 8 UNION ALL SELECT 9),
     Numbers(n) AS
       (SELECT     d5.d * 100000 + d4.d * 10000 + d3.d * 1000 + d2.d * 100 + d1.d * 10 + d0.d
        FROM       Digits AS d0
        CROSS JOIN Digits AS d1
        CROSS JOIN Digits AS d2
        CROSS JOIN Digits AS d3
        CROSS JOIN Digits AS d4
        CROSS JOIN Digits AS d5
        WHERE      d5.d <= 4),
     Letters(l) AS
       (SELECT 'A' UNION ALL SELECT 'C' UNION ALL SELECT 'D' UNION ALL SELECT 'E' UNION ALL SELECT 'G')
INSERT INTO dbo.ProbeInput
           (Key1, Key2)
SELECT      n.n, l.l
FROM        Numbers AS n
CROSS JOIN  Letters AS l;
GO

-- Extra tables for some additional experiments
CREATE TABLE dbo.BuildInputSmall
    (Data1 int      NOT NULL,
     Data2 smallint NOT NULL,
     Data3 tinyint  NOT NULL,
     Data4 tinyint  NOT NULL);

CREATE TABLE dbo.ProbeInputSmall
    (Data1 int      NOT NULL,
     Data2 smallint NOT NULL,
     Data3 tinyint  NOT NULL,
     Data4 tinyint  NOT NULL);
GO
